/*This function sets the initial values of momenta and coordinates
INPUT:
OUTPUT: void
CHANGES: q[i], p[i]
USES: double U, int N, system function gsl-randist
*/

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

extern int N;
extern double U,m_target, sigma_gauss_q, sigma_gauss_p;
extern double *q, *p;

void initial_mom_coor(){

  int flag=0;

  char file_name_gauss_p[100],file_name_gauss_q[100];
  char file_name_gauss_p_1[100],file_name_gauss_q_1[100];

  unsigned int seed;

  double K, V, mx, my;
  int i;
  char sysfile_p[100],sysfile_q[100], temp_dat_p[30], temp_dat_q[30];
  FILE * Mfile_p, * Mfile_q;

  sprintf(file_name_gauss_p,"data_gauss_p_%d.dat", getpid());
  sprintf(file_name_gauss_q,"data_gauss_q_%d.dat", getpid());


  /*
  standard deviaton of initial gaussian distributions
  sigma_gauss_p=.72;
  sigma_gauss_q=.72;

  */

  /*Inicialization of momenta and coordinate vectors:
    We build shell instructions that make use of the mathematical library gsl
    and then execute this commands with the "system" function. The result is
    put in a file and reread to assign it into vectors q[] and p[]*/
  while(flag==0){

    seed=(unsigned int)time( NULL );

    sprintf(sysfile_p,"gsl-randist  %d  %d  gaussian %lf > %s", seed, 2*N, sigma_gauss_p, file_name_gauss_p);

    if(system(sysfile_p)==-1){
      printf("error, system not available\n");
      exit(0);
    }

    sprintf(sysfile_q,"gsl-randist  %d  %d  gaussian %lf > %s", seed+1, N, sigma_gauss_q, file_name_gauss_q);

    if(system(sysfile_q)==-1){
      printf("error, system not available\n");
      exit(0);
    }

    Mfile_p=fopen(file_name_gauss_p,"r");
    if(Mfile_p==NULL){
      printf("Mfile_p -> NULL\n");
      exit(0);
    }

    Mfile_q=fopen(file_name_gauss_q,"r");
    if(Mfile_q==NULL){
      printf("Mfile_q -> NULL\n");
      exit(0);
    }

    K=0.0;
    for(i=1;i<=N;i++) {

      /*both distributions set gaussian*/
      /*fscanf(Mfile_q, "%s\n", &temp_dat_q);
      	q[i]=strtod(temp_dat_q, NULL);*/

      fscanf(Mfile_p, "%s\n", &temp_dat_p);
      q[i]=strtod(temp_dat_p, NULL);

      fscanf(Mfile_p, "%s\n", &temp_dat_p);
      p[i]=strtod(temp_dat_p, NULL);

      /*uncomment for zero initial coordinates*/
      /*q[i] = 0.0;*/

      K+=(0.5*p[i]*p[i]);
    }

    /*Calculation of the potential energy V*/
    mx=0; my=0;
    for(i=1;i<=N;i++) {
      mx+=cos(q[i]);
      my+=sin(q[i]);
    }
    mx/=N; my/=N;

    V = 0.0;
    for(i=1; i<=N; i++) V+=(1-cos(q[i])*mx-sin(q[i])*my);
    V*=0.5;

    /*we rescale the momenta in order to have initial energy U*/
    for(i=1;i<=N;i++) p[i] *= sqrt((U*N-V)/(double)K);

    fclose(Mfile_p); fclose(Mfile_q);

    /*little test*/
    K=0.0;
    for(i=1;i<=N;i++) K+=(0.5*p[i]*p[i]);

    /*Condition must be fullfiled, if not, start again */
    printf("%lf\n",fabs(sqrt(mx*mx+my*my)-m_target)/m_target);
    if(fabs(sqrt(mx*mx+my*my)-m_target)/m_target<0.5){flag=1;printf("yes! :) %d\n", flag);}
    else printf("no :( --> flag %d\n", flag);

  }

  sprintf(file_name_gauss_p_1,"rm -fv %s", file_name_gauss_p);

  if(system(file_name_gauss_p_1)==-1)
    printf("minor error: could not remove data.tmp\ncontinuing...\n");

  sprintf(file_name_gauss_q_1,"rm -fv %s", file_name_gauss_q);

  if(system(file_name_gauss_q_1)==-1)
    printf("minor error: could not remove data.tmp\ncontinuing...\n");

  return;

}
