/*The function dynamics uses Newton law to update the values of momenta and coordinates
INPUT: double tau
OUTPUT: void
CHANGES: double q[i], p[i], mx, my, mx_M, my_M, V, VM, K, KM;
USES: int N, M; double c[], d[], F[];
*/

#include <math.h>

extern int N, M;
extern double *q, *p;
extern double mx, my, mx_M, my_M, V, VM, K, KM, Vi;
extern double *F;

double c[5], d[5];


void dynamics_initialization(){
  /*Integration Constants*/

  c[1] = 1.0/(double)(2.0*(2.0-pow(2.0,1.0/3.0)));
  c[2] = (1.0-pow(2.0,1.0/3.0))*c[1];
  c[3] = c[2];
  c[4] = c[1];

  d[1] = 1.0/(double)(2.0-pow(2.0,1.0/3.0));
  d[2] = -pow(2.0,1.0/3.0)*d[1];
  d[3] = d[1];
  d[4] = 0.;

}

void dynamics(double tau){
/*
   We perform the integration of the trajectories using
   symplectic fourth-order integration method due
   to Neri&Yoshida (Phys. Lett. A 150 (1990) 262-268).

*/
  int s,i,j;

  /*Four loop integration*/
  for(s=1;s<=4;s++){

    /*new coordinates*/
    for(i=1;i<=N;i++) {
      q[i] += c[s]*p[i]*tau;
      /*we want continous border conditions*/
      while(q[i]<0) q[i] += 2.*M_PI;

      q[i] =  fmod(q[i] , 2.*M_PI );
    }
    /*Magnetization (components)*/
    mx=0.; my=0.;
    mx_M=0.; my_M=0.;
    for(i=1;i<=N;i++) {
      mx+=cos(q[i]);  my+=sin(q[i]);
      if(i<=M){
	mx_M+=cos(q[i]); my_M+=sin(q[i]);
      }
    }
    mx/=N; my/=N;
    mx_M/=M; my_M/=M;

    /*forces and potential energy*/
    V = 0.0; VM = 0.0; Vi = 0.0;
    for(i=1; i<=N; i++) {
      F[i] = ( -mx*sin(q[i]) + my*cos(q[i]) );
      /*Only in the fourth loop (s=4) the calculation of the q[]'s is finished*/
      if(s==4){
	V += (1-cos(q[i])*mx-sin(q[i])*my);
	/*Interaction potential*/
	if(i<=M)
	  for(j=M+1;j<=N;j++)
	    Vi+= (1-cos(q[i]-q[j]))/(M*(N-M-1));
      }
    }

    VM=M*(1-mx_M*mx_M-my_M*my_M);
    V*=0.5; VM*=0.5;
    /*new momenta, and kinetic energy*/
    K=0.0;
    KM=0.0;
    for(i=1;i<=N;i++){
      p[i] += d[s]*F[i]*tau;
      /*Only in the fourth loop (s=4) the calculation of the p[]'s is finished*/
      if(s==4){
	if(i<=M) KM+= 0.5*p[i]*p[i];
	K+=0.5*p[i]*p[i];
      }
    }/*end loop i*/
  }/*end of for s*/

  return;

}
