/*The function averages_and_print calculates the averages probabilities of:
 * state, both with a weighed average over the repetitions 
   and with a normal average over the repetitions 
 * energy
 * energy histogram
and prints the results on a file.
It also prints information in an INFO file

INPUT: 

OUTPUT: void

CHANGES: int real_index[], real_index_weighed[]; 
double P_state_weighed[], P_state[], PP_state[], g_med[]; 
double P_energy[], PP_energy[], P_energy_histo[];

USES: int E_dim;
int g_index[], occup_index[], occup_index_histo[]; long unsigned T;
char file_name[], file_name1[];  
*/

#include <stdio.h>
#include <time.h>

extern int N, M, E_dim, VEZ, vez;
/*This is defined in make_occup_key():*/
extern int *real_K_index, *real_index;
extern int *occup_index, *occup_index_histo, *occup_K_index, *occup_K_index_histo;
extern long unsigned T, t, Ttrans;
extern double U, DE;
extern double *P_K_energy, *P_K_energy_histo, *P_energy, *PP_energy,
  *P_energy_histo, *E_index;
extern char file_name_K[], file_name[], file_name1[];
extern time_t tmpi, tmpf;


void averages_and_print(){

  int i;
  unsigned long occup_total;
  double stddev_tmp;
  FILE *infonline, *file_occup, *file_occup_K;

  file_occup=fopen(file_name,"a");
  fprintf(file_occup,"\n#run %d/%d T/Ttrans:%d/%d\n",vez,VEZ,T,Ttrans);
  fprintf(file_occup,"#We are here!! run #%d\t, T=%d\n",vez,T);
  fprintf(file_occup, "#index #E #P_energy #stdv_P_energy #counts #counts_acc #P_energy_histo\n");


  for(i=0; i<=E_dim-1; i++){

    if(occup_index[i]>0){
      real_index[i]++;

      P_energy[i]=((real_index[i]-1)*P_energy[i]+(occup_index[i]/(double)T))/real_index[i];

      PP_energy[i]=((real_index[i]-1)*PP_energy[i]+
		    (occup_index[i]/(double)T)*(occup_index[i]/(double)T))/real_index[i];

    }

    if(occup_index_histo[i]>0){
      P_energy_histo[i]=occup_index_histo[i]/(double)(T*vez);
    }

    /*Write out occupation numbers*/
    stddev_tmp=PP_energy[i]-P_energy[i]*P_energy[i];
    if(stddev_tmp<0)stddev_tmp*=-1.0;

    if(P_energy_histo[i]!=0.)fprintf(file_occup, "%u %.3lf %.8lf %.8lf %.8lf %.8u %.8lf\n",
				     i,  E_index[i], P_energy[i], sqrt(stddev_tmp)/2.0,
				     (occup_index[i]/(double)T), real_index[i],
				     P_energy_histo[i]);
    occup_index[i]=0;

  }
  fclose(file_occup);

  file_occup_K=fopen(file_name_K,"a");
  fprintf(file_occup_K,"\n#run %d/%d T/Ttrans:%d/%d\n",vez,VEZ,T,Ttrans);
  fprintf(file_occup_K,"#Im here!! run #%d\t, T=%d\n",vez,T);
  fprintf(file_occup_K,"#index #E #P_K_energy #counts #counts_acc #P_K_energy_histo\n");


  for(i=0; i<=E_dim-1; i++){

    /*Same thing for K quantities*/
    if(occup_K_index[i]>0){
      real_K_index[i]++;
      P_K_energy[i]=((real_K_index[i]-1)*P_K_energy[i]+(occup_K_index[i]/(double)T))/real_K_index[i];
    }

    if(occup_K_index_histo[i]>0)  P_K_energy_histo[i]=occup_K_index_histo[i]/(double)(T*vez);

    if(P_K_energy_histo[i]!=0.)fprintf(file_occup_K, "%u %.3lf %.8lf %.8lf %.8u %.8lf\n",
		i,  E_index[i], P_K_energy[i], (occup_K_index[i]/(double)T),
		real_K_index[i], P_K_energy_histo[i]);

    occup_K_index[i]=0;

  }
  fclose(file_occup_K);


  /* We construct a file to have every parameter and other useful information*/
  infonline=fopen(file_name1,"w");

  fprintf(infonline, "N %d M %d -- t %d/%d vez %d/%d\n", N, M, t, T, vez, VEZ);
  fprintf(infonline, "U %lf, Ttrans %d\n", U, Ttrans);
  fprintf(infonline, "Others: DE %lf \n", DE);
  /*Elapsed time*/
  tmpf=time(&tmpf);
  fprintf(infonline,"Present time: %s\nRunning time: %.2lf s (%.3lf min)\n",
	  ctime(&tmpf), difftime(tmpf,tmpi), difftime(tmpf,tmpi)/60);
  if(vez!=VEZ || t!=T)
    fprintf(infonline,"Estimated time: %.2lf s (%.2lf min)\n",
	    difftime(tmpf,tmpi)/(vez*T)*(VEZ-vez)*T,
	    difftime(tmpf,tmpi)/(vez*T)*(VEZ-vez)*T/60);

  fclose(infonline);

  return;

}
